@@ -57,6 +57,7 @@ namespace common {
 #define CCL_ALLREDUCE "CCL_ALLREDUCE"
 #define CCL_ALLGATHER "CCL_ALLGATHER"
 #define CCL_BCAST "CCL_BCAST"
+#define CCL_ALLTOALL "CCL_ALLTOALL"
 #define GLOO_ALLREDUCE "GLOO_ALLREDUCE"
 #define GLOO_ALLGATHER "GLOO_ALLGATHER"
 #define GLOO_BCAST "GLOO_BCAST"
@@ -83,6 +84,7 @@ namespace common {
 #define HOROVOD_NUM_NCCL_STREAMS "HOROVOD_NUM_NCCL_STREAMS"
 #define HOROVOD_CPU_OPERATIONS "HOROVOD_CPU_OPERATIONS"
 #define HOROVOD_CONTROLLER "HOROVOD_CONTROLLER"
+#define HOROVOD_CCL_CACHE "HOROVOD_CCL_CACHE"
 #define HOROVOD_GLOO_IFACE "HOROVOD_GLOO_IFACE"
 #define HOROVOD_MPI "MPI"
 #define HOROVOD_CCL "CCL"
