@@ -27,8 +27,9 @@ class GlooController : public Controller {
 public:
   GlooController(ResponseCache& response_cache, TensorQueue& tensor_queue,
                  Timeline& timeline, ParameterManager& parameter_manager,
-                 GlooContext& gloo_context)
-      : Controller(response_cache, tensor_queue, timeline, parameter_manager),
+                 GroupTable& group_table, GlooContext& gloo_context)
+      : Controller(response_cache, tensor_queue, timeline, parameter_manager,
+                   group_table),
         gloo_context_(gloo_context) {};
 
   int GetTypeSize(DataType dtype) override;
