@@ -13,30 +13,32 @@
 # limitations under the License.
 # ==============================================================================
 
-from horovod.common.util import check_extension
+from horovod.common.util import check_extension, split_list
 
 check_extension('horovod.mxnet', 'HOROVOD_WITH_MXNET',
                 __file__, 'mpi_lib')
 
 from horovod.mxnet.functions import allgather_object, broadcast_object
 from horovod.mxnet.mpi_ops import allgather
-from horovod.mxnet.mpi_ops import allreduce, allreduce_
+from horovod.mxnet.mpi_ops import allreduce, allreduce_, grouped_allreduce, grouped_allreduce_
 from horovod.mxnet.mpi_ops import alltoall
 from horovod.mxnet.mpi_ops import broadcast, broadcast_
 from horovod.mxnet.mpi_ops import init, shutdown
+from horovod.mxnet.mpi_ops import is_initialized, start_timeline, stop_timeline
 from horovod.mxnet.mpi_ops import size, local_size, rank, local_rank
 from horovod.mxnet.mpi_ops import mpi_threads_supported, mpi_enabled, mpi_built
 from horovod.mxnet.mpi_ops import gloo_enabled, gloo_built
 from horovod.mxnet.mpi_ops import nccl_built, ddl_built, ccl_built, cuda_built, rocm_built
 
 import mxnet as mx
+from collections import OrderedDict, defaultdict
 import types
 import warnings
 
 
 # This is where Horovod's DistributedOptimizer wrapper for MXNet goes
 class DistributedOptimizer(mx.optimizer.Optimizer):
-    def __init__(self, optimizer, gradient_predivide_factor=1.0):
+    def __init__(self, optimizer, gradient_predivide_factor=1.0, num_groups=0):
         if gradient_predivide_factor != 1.0 and rocm_built():
             raise ValueError('gradient_predivide_factor not supported yet with ROCm')
 
@@ -45,6 +47,7 @@ class DistributedOptimizer(mx.optimizer.Optimizer):
         # performing average in allreduce, has better performance.
         self._optimizer.rescale_grad *= (gradient_predivide_factor / size())
         self._gradient_predivide_factor = gradient_predivide_factor
+        self._num_groups = num_groups
 
     def __getattr__(self, item):
         return getattr(self._optimizer, item)
@@ -56,10 +59,18 @@ class DistributedOptimizer(mx.optimizer.Optimizer):
         if size() == 1: return
 
         if isinstance(index, (tuple, list)):
-            for i in range(len(index)):
-                allreduce_(grad[i], average=False,
-                           name=str(index[i]), priority=-i,
-                           prescale_factor=1.0 / self._gradient_predivide_factor)
+            if (self._num_groups > 0):
+                grad_split = split_list(grad, self._num_groups)
+                index_split = split_list(index, self._num_groups)
+
+                for i, (grads, indices) in enumerate(zip(grad_split, index_split)):
+                    grouped_allreduce_(tensors=grads, average=False, name="{}:{}".format(indices[0], indices[-1]), priority=-i,
+                                       prescale_factor=1.0 / self._gradient_predivide_factor)
+            else:
+              for i in range(len(index)):
+                  allreduce_(grad[i], average=False,
+                             name=str(index[i]), priority=-i,
+                             prescale_factor=1.0 / self._gradient_predivide_factor)
         else:
             allreduce_(grad, average=False, name=str(index),
                        prescale_factor=1.0 / self._gradient_predivide_factor)
@@ -89,16 +100,38 @@ class DistributedOptimizer(mx.optimizer.Optimizer):
 # 2. DistributedTrainer performs allreduce(summation) and average
 #    while Trainer only performs allreduce(summation).
 class DistributedTrainer(mx.gluon.Trainer):
+    """The distributed trainer for data parallel training.
+
+    Arguments:
+        params: dict of parameters to train
+        optimizer: mx.optim.Optimizer. the choice of optimizer
+        optimizer_params: hyper-parameter of the chosen optimizer
+        gradient_predivide_factor: gradient_predivide_factor splits the averaging
+              before and after the sum. Gradients are scaled by
+              1.0 / gradient_predivide_factor before the sum and
+              gradient_predivide_factor / size after the sum.
+        prefix: the prefix of the parameters this trainer manages.
+              If multiple trainers are used in the same program,
+              they must be specified by different prefixes to avoid tensor name collision.
+    """
     def __init__(self, params, optimizer, optimizer_params=None,
-                 gradient_predivide_factor=1.0):
+                 gradient_predivide_factor=1.0, prefix=None,
+                 num_groups=0):
         if gradient_predivide_factor != 1.0 and rocm_built():
             raise ValueError('gradient_predivide_factor not supported yet with ROCm')
-
         if isinstance(optimizer, DistributedOptimizer):
             optimizer = optimizer._optimizer
             warnings.warn("DistributedTrainer does not take DistributedOptimizer "
                           "as its optimizer. We have unwrapped it for you.")
 
+        # To ensure consistent parameter ordering across workers, sort params before
+        # passing to base Trainer constructor. This logic is consistent with trainer.py
+        # since v1.6 but we do it here for backwards compatability
+        if isinstance(params, dict):
+            params = OrderedDict(params)
+        elif isinstance(params, (list, tuple)):
+            params = sorted(params)
+
         super(DistributedTrainer, self).__init__(
             params, optimizer, optimizer_params=optimizer_params, kvstore=None)
 
@@ -107,29 +140,56 @@ class DistributedTrainer(mx.gluon.Trainer):
         # average in allreduce, has better performance. 
         self._scale *= (gradient_predivide_factor / size())
         self._gradient_predivide_factor = gradient_predivide_factor
+        assert prefix is None or isinstance(prefix, str)
+        self._prefix = prefix if prefix else ""
+        self._num_groups = num_groups
 
     def _allreduce_grads(self):
         if size() == 1: return
 
-        for i, param in enumerate(self._params):
-            if param.grad_req != 'null':
-                allreduce_(param.list_grad()[0], average=False,
-                           name=param.name, priority=-i,
-                           prescale_factor=1.0 / self._gradient_predivide_factor)
+        if (self._num_groups > 0):
+            grads = []
+            names = []
 
+            for i, param in enumerate(self._params):
+                if param.grad_req != 'null':
+                    grads.append(param.list_grad()[0])
+                    names.append(self._prefix + str(i))
+
+            grads_split = split_list(grads, self._num_groups)
+            names_split = split_list(names, self._num_groups)
+
+            for i, (group_grads, group_names) in enumerate(zip(grads_split, names_split)):
+                # For better performance, enqueue groups in separate grouped_allreduce calls by dtype.
+                entries_by_dtype = defaultdict(list)
+                for grad, name in zip(group_grads, group_names):
+                    entries_by_dtype[grad.dtype].append((grad, name))
+
+                for entries in entries_by_dtype.values():
+                    grads, names = zip(*entries)
+                    grouped_allreduce_(tensors=grads, average=False, name="{}:{}".format(names[0], names[-1]), priority=-i,
+                                       prescale_factor=1.0 / self._gradient_predivide_factor)
+        else:
+            # In MXNet 2.0, param.name is no longer unique.
+            # Meanwhile, since horovod requires Python 3.6, there is no need to sort
+            # self._params as enumerating a python dict is always deterministic.
+            for i, param in enumerate(self._params):
+                if param.grad_req != 'null':
+                    allreduce_(param.list_grad()[0], average=False,
+                               name=self._prefix + str(i), priority=-i,
+                               prescale_factor=1.0 / self._gradient_predivide_factor)
 
 # Wrapper to inject Horovod broadcast after parameter initialization
-def _append_broadcast_init(param, root_rank):
+def _append_broadcast_init(param, root_rank, name):
     init_impl = getattr(param, '_init_impl')
     def wrapped_init_impl(self, *args, **kwargs):
         init_impl(*args, **kwargs)
-        broadcast_(self.data(), root_rank=root_rank, name=self.name)
+        broadcast_(self.data(), root_rank=root_rank, name=name)
     return wrapped_init_impl
 
 
-def broadcast_parameters(params, root_rank=0):
-    """
-    Broadcasts the parameters from root rank to all other processes.
+def broadcast_parameters(params, root_rank=0, prefix=None):
+    """Broadcasts the parameters from root rank to all other processes.
     Typical usage is to broadcast the `Module.get_params()` or the
     `Block.collect_params()`.
 
@@ -139,26 +199,39 @@ def broadcast_parameters(params, root_rank=0):
             - ParameterDict to broadcast
         root_rank: The rank of the process from which parameters will be
                    broadcasted to all other processes.
+        prefix: The prefix of the parameters to broadcast.
+              If multiple `broadcast_parameters` are called in the same program,
+              they must be specified by different prefixes to avoid tensor name collision.
     """
     if size() == 1: return
 
     tensors = []
     names = []
-    if isinstance(params, dict):
-        names, tensors = zip(*params.items())
-    elif isinstance(params, mx.gluon.parameter.ParameterDict):
+    assert prefix is None or isinstance(prefix, str)
+    prefix = prefix if prefix else ""
+    try:
+        from mxnet.gluon.parameter import ParameterDict
+        valid_types = (dict, ParameterDict)
+    except ImportError:
+        valid_types = (dict,)
+    if isinstance(params, valid_types):
         for name, p in sorted(params.items()):
             try:
-                tensors.append(p.data())
-                names.append(name)
+                if isinstance(p, mx.gluon.parameter.Parameter):
+                    tensors.append(p.data())
+                else:
+                    tensors.append(p)
+                names.append(prefix + str(name))
             except mx.gluon.parameter.DeferredInitializationError:
                 # Inject wrapper method with post-initialization broadcast to
                 # handle parameters with deferred initialization
-                new_init = _append_broadcast_init(p, root_rank)
+                # we use the key of params instead of param.name, since
+                # param.name is no longer unique in MXNet 2.0
+                new_init = _append_broadcast_init(p, root_rank, prefix + str(name))
                 p._init_impl = types.MethodType(new_init, p)
     else:
         raise ValueError('invalid params of type: %s' % type(params))
 
     # Run broadcasts.
     for tensor, name in zip(tensors, names):
-        broadcast_(tensor, root_rank, name=str(name))
+        broadcast_(tensor, root_rank, name=name)
