@@ -58,6 +58,8 @@ class EstimatorParams(Params):
     callbacks = Param(Params._dummy(), 'callbacks', 'callbacks')
     batch_size = Param(Params._dummy(), 'batch_size', 'batch size',
                        typeConverter=TypeConverters.toInt)
+    val_batch_size = Param(Params._dummy(), 'val_batch_size', 'validation batch size',
+                           typeConverter=TypeConverters.toInt)
     epochs = Param(Params._dummy(), 'epochs', 'epochs', typeConverter=TypeConverters.toInt)
     train_steps_per_epoch = Param(Params._dummy(), 'train_steps_per_epoch',
                                   'number of training (batches) steps per epoch',
@@ -109,6 +111,7 @@ class EstimatorParams(Params):
             gradient_compression=None,
             compress_sparse_cols=False,
             batch_size=32,
+            val_batch_size=None,
             epochs=1,
             verbose=1,
             callbacks=[],
@@ -223,6 +226,12 @@ class EstimatorParams(Params):
     def getBatchSize(self):
         return self.getOrDefault(self.batch_size)
 
+    def setValBatchSize(self, value):
+        return self._set(val_batch_size=value)
+
+    def getValBatchSize(self):
+        return self.getOrDefault(self.val_batch_size)
+
     def setEpochs(self, value):
         return self._set(epochs=value)
 
