@@ -148,6 +148,7 @@ public:
   virtual const common::TensorShape shape() const override;
   virtual const void* data() const override;
   virtual int64_t size() const override;
+  const ::tensorflow::Tensor* tensor() const;
 
 protected:
   ::tensorflow::Tensor tensor_;
@@ -258,6 +259,8 @@ const void* TFTensor::data() const { return (const void*)tensor_.tensor_data().d
 
 int64_t TFTensor::size() const { return (int64_t)tensor_.tensor_data().size(); }
 
+const ::tensorflow::Tensor*  TFTensor::tensor() const { return &tensor_; }
+
 TFOpContext::TFOpContext(OpKernelContext* context) : context_(context) {}
 
 common::Status TFOpContext::AllocatePersistent(
@@ -375,6 +378,7 @@ public:
     OP_REQUIRES_OK(context, context->GetAttr("reduce_op", &reduce_op_));
     OP_REQUIRES_OK(context, context->GetAttr("prescale_factor", &prescale_factor_));
     OP_REQUIRES_OK(context, context->GetAttr("postscale_factor", &postscale_factor_));
+    OP_REQUIRES_OK(context, context->GetAttr("ignore_name_scope", &ignore_name_scope_));
   }
 
   void ComputeAsync(OpKernelContext* context, DoneCallback done) override {
@@ -382,6 +386,12 @@ public:
                          done);
 
     auto node_name = name();
+    if (ignore_name_scope_) {
+      auto pos = node_name.find_last_of('/');
+      if (pos != std::string::npos) {
+        node_name = node_name.substr(pos + 1);
+      }
+    }
     auto device = GetDeviceID(context);
     auto tensor = context->input(0);
     horovod::common::ReduceOp reduce_op = static_cast<horovod::common::ReduceOp>(reduce_op_);
@@ -407,6 +417,7 @@ private:
   // Using float since TF does not support double OP attributes
   float prescale_factor_;
   float postscale_factor_;
+  bool ignore_name_scope_;
 };
 
 REGISTER_KERNEL_BUILDER(Name("HorovodAllreduce").Device(DEVICE_CPU),
@@ -421,6 +432,7 @@ REGISTER_OP("HorovodAllreduce")
     .Attr("reduce_op: int")
     .Attr("prescale_factor: float")
     .Attr("postscale_factor: float")
+    .Attr("ignore_name_scope: bool = False")
     .Input("tensor: T")
     .Output("sum: T")
     .SetShapeFn([](shape_inference::InferenceContext* c) {
@@ -440,16 +452,140 @@ Output
     sum:    A tensor with the same shape as `tensor`, summed across all MPI processes.
 )doc");
 
-class HorovodAllgatherOp : public AsyncOpKernel {
+class HorovodGroupedAllreduceOp : public AsyncOpKernel {
 public:
-  explicit HorovodAllgatherOp(OpKernelConstruction* context)
-      : AsyncOpKernel(context) {}
+  explicit HorovodGroupedAllreduceOp(OpKernelConstruction* context)
+      : AsyncOpKernel(context) {
+    OP_REQUIRES_OK(context, context->GetAttr("reduce_op", &reduce_op_));
+    OP_REQUIRES_OK(context, context->GetAttr("prescale_factor", &prescale_factor_));
+    OP_REQUIRES_OK(context, context->GetAttr("postscale_factor", &postscale_factor_));
+    OP_REQUIRES_OK(context, context->GetAttr("ignore_name_scope", &ignore_name_scope_));
+    OP_REQUIRES_OK(context, context->GetAttr("num_tensors", &num_tensors_));
+  }
 
   void ComputeAsync(OpKernelContext* context, DoneCallback done) override {
     OP_REQUIRES_OK_ASYNC(context, ConvertStatus(common::CheckInitialized()),
                          done);
 
     auto node_name = name();
+    if (ignore_name_scope_) {
+      auto pos = node_name.find_last_of('/');
+      if (pos != std::string::npos) {
+        node_name = node_name.substr(pos + 1);
+      }
+    }
+    auto device = GetDeviceID(context);
+    horovod::common::ReduceOp reduce_op = static_cast<horovod::common::ReduceOp>(reduce_op_);
+    std::vector<Tensor*> outputs(num_tensors_);
+
+    std::vector<std::shared_ptr<common::ReadyEvent>> ready_events;
+    std::vector<std::shared_ptr<common::OpContext>> hvd_contexts;
+    std::vector<std::shared_ptr<common::Tensor>> hvd_tensors;
+    std::vector<std::shared_ptr<common::Tensor>> hvd_outputs;
+    std::vector<common::StatusCallback> callbacks;
+    std::vector<std::string> names;
+    ready_events.reserve(num_tensors_);
+    hvd_contexts.reserve(num_tensors_);
+    hvd_tensors.reserve(num_tensors_);
+    hvd_outputs.reserve(num_tensors_);
+    callbacks.reserve(num_tensors_);
+    names.reserve(num_tensors_);
+    auto callback_mutex = std::make_shared<std::mutex>();
+    auto callback_count = std::make_shared<int>(0);
+    int num_tensors = num_tensors_;
+
+    for (int i = 0; i < num_tensors_; ++i) {
+        auto tensor = context->input(i);
+        OP_REQUIRES_OK_ASYNC(
+            context, context->allocate_output(i, tensor.shape(), &outputs[i]), done);
+        // ReadyEvent makes sure input tensor is ready, and output is allocated.
+        ready_events.emplace_back(std::shared_ptr<common::ReadyEvent>(RecordReadyEvent(context)));
+        hvd_contexts.emplace_back(std::make_shared<TFOpContext>(context));
+        hvd_tensors.emplace_back(std::make_shared<TFTensor>(tensor));
+        names.emplace_back(node_name + "_" + std::to_string(i+1) + "of" + std::to_string(num_tensors));
+        hvd_outputs.emplace_back(std::make_shared<TFTensor>(*outputs[i]));
+        callbacks.emplace_back(
+            [context, done, callback_mutex, callback_count, num_tensors](const common::Status& status) mutable {
+                // Must only invoke callback on last tensor.
+                std::lock_guard<std::mutex> guard(*callback_mutex);
+                (*callback_count)++;
+                if (*callback_count == num_tensors) {
+                    context->SetStatus(ConvertStatus(status));
+                    done();
+                }
+            }
+        );
+    }
+
+    auto enqueue_result = EnqueueTensorAllreduces(
+        hvd_contexts, hvd_tensors, hvd_outputs, ready_events, names, device,
+        callbacks, reduce_op, (double) prescale_factor_, (double) postscale_factor_);
+    OP_REQUIRES_OK_ASYNC(context, ConvertStatus(enqueue_result), done);
+  }
+
+private:
+  int reduce_op_;
+  // Using float since TF does not support double OP attributes
+  float prescale_factor_;
+  float postscale_factor_;
+  bool ignore_name_scope_;
+  int num_tensors_;
+};
+
+REGISTER_KERNEL_BUILDER(Name("HorovodGroupedAllreduce").Device(DEVICE_CPU),
+                        HorovodGroupedAllreduceOp);
+#if HOROVOD_GPU_ALLREDUCE
+REGISTER_KERNEL_BUILDER(Name("HorovodGroupedAllreduce").Device(DEVICE_GPU),
+                        HorovodGroupedAllreduceOp);
+#endif
+
+REGISTER_OP("HorovodGroupedAllreduce")
+    .Attr("T: {int32, int64, float16, float32, float64}")
+    .Attr("reduce_op: int")
+    .Attr("prescale_factor: float")
+    .Attr("postscale_factor: float")
+    .Attr("ignore_name_scope: bool = False")
+    .Attr("num_tensors: int")
+    .Input("tensors: num_tensors*T")
+    .Output("sum: num_tensors*T")
+    .SetShapeFn([](shape_inference::InferenceContext* c) {
+      int num_tensors;
+      for (int i = 0; i < c->num_inputs(); ++i) {
+          c->set_output(i, c->input(i));
+      }
+      return Status::OK();
+    })
+    .Doc(R"doc(
+Perform an MPI Allreduce on a list tensors. All other processes that do a reduction
+on a tensor with the same name must have the same dimension for that tensor.
+Tensors are reduced with other tensors that have the same node name for the
+allreduce.
+
+Arguments
+    tensors:     A list of tensors to reduce.
+
+Output
+    sum:    A list of tensors with the same shape as corresponding tensors in `tensors`, summed across all MPI processes.
+)doc");
+
+class HorovodAllgatherOp : public AsyncOpKernel {
+public:
+  explicit HorovodAllgatherOp(OpKernelConstruction* context)
+      : AsyncOpKernel(context) {
+    OP_REQUIRES_OK(context, context->GetAttr("ignore_name_scope", &ignore_name_scope_));
+  }
+
+  void ComputeAsync(OpKernelContext* context, DoneCallback done) override {
+    OP_REQUIRES_OK_ASYNC(context, ConvertStatus(common::CheckInitialized()),
+                         done);
+
+    auto node_name = name();
+    if (ignore_name_scope_) {
+      auto pos = node_name.find_last_of('/');
+      if (pos != std::string::npos) {
+        node_name = node_name.substr(pos + 1);
+      }
+    }
     auto device = GetDeviceID(context);
     auto tensor = context->input(0);
     // ReadyEvent makes sure input tensor is ready.  We cannot pre-allocate
@@ -466,6 +602,9 @@ public:
         });
     OP_REQUIRES_OK_ASYNC(context, ConvertStatus(enqueue_result), done);
   }
+
+private:
+  bool ignore_name_scope_;
 };
 
 REGISTER_KERNEL_BUILDER(Name("HorovodAllgather").Device(DEVICE_CPU),
@@ -478,6 +617,7 @@ REGISTER_KERNEL_BUILDER(Name("HorovodAllgather").Device(DEVICE_GPU),
 REGISTER_OP("HorovodAllgather")
     .Attr(
         "T: {uint8, int8, uint16, int16, int32, int64, float16, float32, float64, bool}")
+    .Attr("ignore_name_scope: bool = False")
     .Input("tensor: T")
     .Output("output: T")
     .SetShapeFn([](shape_inference::InferenceContext* c) {
@@ -504,6 +644,7 @@ public:
   explicit HorovodBroadcastOp(OpKernelConstruction* context)
       : AsyncOpKernel(context) {
     OP_REQUIRES_OK(context, context->GetAttr("root_rank", &root_rank_));
+    OP_REQUIRES_OK(context, context->GetAttr("ignore_name_scope", &ignore_name_scope_));
   }
 
   void ComputeAsync(OpKernelContext* context, DoneCallback done) override {
@@ -511,6 +652,12 @@ public:
                          done);
 
     auto node_name = name();
+    if (ignore_name_scope_) {
+      auto pos = node_name.find_last_of('/');
+      if (pos != std::string::npos) {
+        node_name = node_name.substr(pos + 1);
+      }
+    }
     auto device = GetDeviceID(context);
     auto tensor = context->input(0);
     Tensor* output = nullptr;
@@ -539,6 +686,7 @@ public:
 
 private:
   int root_rank_;
+  bool ignore_name_scope_;
 };
 
 REGISTER_KERNEL_BUILDER(Name("HorovodBroadcast").Device(DEVICE_CPU),
@@ -552,6 +700,7 @@ REGISTER_OP("HorovodBroadcast")
     .Attr(
         "T: {uint8, int8, uint16, int16, int32, int64, float16, float32, float64, bool}")
     .Attr("root_rank: int")
+    .Attr("ignore_name_scope: bool = False")
     .Input("tensor: T")
     .Output("output: T")
     .SetShapeFn([](shape_inference::InferenceContext* c) {
@@ -724,13 +873,21 @@ Output
 class HorovodAlltoallOp : public AsyncOpKernel {
 public:
   explicit HorovodAlltoallOp(OpKernelConstruction* context)
-      : AsyncOpKernel(context) {}
+      : AsyncOpKernel(context) {
+    OP_REQUIRES_OK(context, context->GetAttr("ignore_name_scope", &ignore_name_scope_));
+  }
 
   void ComputeAsync(OpKernelContext* context, DoneCallback done) override {
     OP_REQUIRES_OK_ASYNC(context, ConvertStatus(common::CheckInitialized()),
                          done);
 
     auto node_name = name();
+    if (ignore_name_scope_) {
+      auto pos = node_name.find_last_of('/');
+      if (pos != std::string::npos) {
+        node_name = node_name.substr(pos + 1);
+      }
+    }
     auto device = GetDeviceID(context);
     auto tensor = context->input(0);
     auto splits = context->input(1);
@@ -746,6 +903,8 @@ public:
         });
     OP_REQUIRES_OK_ASYNC(context, ConvertStatus(enqueue_result), done);
   }
+private:
+  bool ignore_name_scope_;
 }; // namespace tensorflow
 
 REGISTER_KERNEL_BUILDER(Name("HorovodAlltoall").Device(DEVICE_CPU),
@@ -758,6 +917,7 @@ REGISTER_KERNEL_BUILDER(Name("HorovodAlltoall").Device(DEVICE_GPU).HostMemory("s
 REGISTER_OP("HorovodAlltoall")
     .Attr(
         "T: {uint8, int8, uint16, int16, int32, int64, float16, float32, float64, bool}")
+    .Attr("ignore_name_scope: bool = False")
     .Input("tensor: T")
     .Input("splits: int32")
     .Output("output: T")
